require 'json'
require 'uu/os/rest/future'
require 'uu/os/uesuri'
require 'uu/os/uesuri_builder'

module UU
  module OS
    module VUC
      # Form navigation object.
      class Navigation

        # Raw navigation attributes.
        # @private
        attr_reader :raw_attrs

        # Raw navigation attributes.
        # (Temporarily re-added for backward compatibility, see 01RHF02-HP003)
        # @private
        attr_reader :attributes

        # Navigation action. Can be UESURI or value from {UU::OS::VUC::Navigation::NavigationAction}.
        # @param [String, UU::OS::UESURI]
        # @return [String, UU::OS::UESURI]
        attr_accessor :action

        # Progress dialog settings (progress must be enabled via action).
        # @return [UU::OS::VUC::Navigation::Progress]
        attr_reader :progress

        # Creates new instance of navigation.
        #
        # @param [String, Hash] data Initialization data in form of JSON string or Hash (optional)
        def initialize(data = nil)
          if data.nil?
            @raw_attrs = {}
            return
          end
          if data.kind_of?String
            data = JSON.parse(data, :symbolize_names => true)
          end
          if data.kind_of?Hash
            @raw_attrs = data
          elsif data.kind_of?Navigation
            if data.raw_attrs
              @raw_attrs = data.raw_attrs
            else
              @raw_attrs = {}
            end
          else
            raise ArgumentError.new("Navigation data must be JSON String or Hash, but was #{data.class}.")
          end
        end

        def raw_attrs
          # Disallow manipulating with raw attributes
          @raw_attrs.clone if @raw_attrs
        end

        def attributes
          @raw_attrs
        end

        def action
          @raw_attrs[:action]
        end

        def action=(action)
          @raw_attrs[:action] = action
        end

        def progress
          if @raw_attrs[:progress].nil?
            @raw_attrs[:progress] = {}
            @progress = Progress.new(@raw_attrs[:progress])
          end
          return @progress
        end

        # Creates action for launching download from client (browser).
        #
        # @example
        #   # end VUC event processing by downloading an attachment
        #   view.navigation.action = UU::OS::VUC::Navigation.create_download_action('ues:TER:ART:ATTACHMENT')
        #
        # @param download_uri [String, UU::OS::UESURI] The UES-URI which will end with download when used.
        # @return [String] Action (URI) for launching download.
        def self.create_download_action(download_uri)
          if download_uri.nil? || (download_uri.kind_of?(String) && download_uri.empty?)
            return download_uri
          end
          unless (download_uri.kind_of?(String) && (download_uri.start_with?('ues:') || download_uri.start_with?('uu-uds:'))) || download_uri.kind_of?(UU::OS::UESURI)
            raise ArgumentError.new("download_uri must be UES-URI instance or URI string containing UES-URI or UDS-URI, but it was #{download_uri}.")
          end
          uri = UU::OS::UESURI.new(download_uri)
          p = []
          p = uri.parameters unless uri.parameters.nil?
          win_params = p.drop_while { |it| it.name != 'windowParameters' }.first
          if !win_params.nil? && win_params.value.start_with?('{')
            win_params = JSON.parse(win_params.value, :symbolize_names => true)
          else
            win_params = {}
          end
          win_params[:target] = 'DOWNLOAD'
          action_uri_b = UU::OS::UESURIBuilder.parse_uesuri(uri).clear_parameters()
          p.each { |it| action_uri_b.add_parameter(it.name, it.value) unless it.name == 'windowParameters' }
          action_uri_b.add_parameter('windowParameters', win_params.to_json)
          uri.schema + ':' + action_uri_b.to_uesuri.to_s.gsub(/^ues:/, "") # UESURIBuilder currently forces scheme to 'ues:' but the initial URI might have different one, e.g. uu-uds:
        end

        # Progress dialog configuration.
        class Progress

          # Raw configuration attributes.
          # @private
          attr_reader :raw_attrs

          # UESURI of asynchronous process.
          # @param [String, UU::OS::UESURI]
          # @return [String, UU::OS::UESURI]
          attr_accessor :process_uri

          # Code of process event providing result.
          # @param [String]
          # @return [String]
          attr_accessor :result_code

          # Type of progress dialog.
          # @param [String]
          # @return [String]
          attr_accessor :dialog_type

          # @private
          def initialize(data)
            @raw_attrs = data
          end

          def raw_attrs
            # Disallow manipulating with raw attributes
            @raw_attrs.clone if @raw_attrs
          end

          def process_uri
            @raw_attrs[:processUri]
          end

          def process_uri=(process_uri)

            # because of backward compatibility we need Progress.process_uri to
            # accept the Future as well as the UESURI
            if process_uri.kind_of?UU::OS::REST::Future
              process_uri = process_uri.get_process_uri
            end
            @raw_attrs[:processUri] = process_uri
          end

          def result_code
            @raw_attrs[:resultCode]
          end

          def result_code=(result_code)
            @raw_attrs[:resultCode] = result_code
          end

          def dialog_type
            @raw_attrs[:dialogType]
          end

          def dialog_type=(dialog_type)
            @raw_attrs[:dialogType] = dialog_type
          end

        end

        # Enumeration with system predefined navigation actions.
        class NavigationAction

          # Navigate to previous use case.
          GO_BACK = 'uu.os.vuc.Navigation.GO_BACK'

          # Order form to use progress dialog.
          WAIT_ON_PROGRESS = 'uu.os.vuc.Navigation.WAIT_ON_PROGRESS'

          # Execute super use case and terminate the current use case. Currently,
          # this option only works for an application use case which overrides
          # a system use case.
          EXECUTE_SUPER_UC = 'uu.os.vuc.Navigation.EXECUTE_SUPER_UC'

        end

      end

    end
  end
end

module UU
  module ADK

    # These constants may be already used within controllers.
    # We have to maintain backward compatibility as much as possible.
    # @private
    # @deprecated
    class Navigation

      # @private
      # @deprecated
      ACTION_GO_BACK = UU::OS::VUC::Navigation::NavigationAction::GO_BACK

      # @private
      # @deprecated
      ACTION_WAIT_ON_PROGRESS = UU::OS::VUC::Navigation::NavigationAction::WAIT_ON_PROGRESS

    end
  end
end
