module UU
  module OS
    module VUC
      
      # View template that can be used for setting custom view.
      class ViewTemplate

        # Raw context attributes.
        # @private
        attr_reader :raw_attrs


        # UESURI of sheet that will be used as form template. Default value is the main sheet of registered VUC. 
        # You can use sheets from this VUC only, it's means that only sheets related to VUC artifact (from parameter context.use_case_realization_uri) are allowed,
        # otherwise form execution fails with error. If you set non-exists uri than form execution will fail as well.
        #  
        # @note This parameters can be set only in on_before_init method, if you set data_uri in another method, it will have no effect. 
        # 
        # @example
        #   # uri of default VUC
        #   UCR_URI = "ues:UNI:CDS.GUESTBOOK/COMMENT/CREATE"
        #   
        #   # code of sheet with extended form
        #   EXTENDED_FORM_CODE = "EXTENDED_FORM"
        #   
        #   def on_before_init(ctx)
        #   
        #     # get default view template => main sheet on VUC
        #     view_template_uri = ctx.view_template.data_uri # String ues:UNI:CDS.GUESTBOOK/COMMENT/CREATE:DEFAULT_FORM
        #   
        #     use_extended_form = ... # some logic to find out if use can see extended form
        #   
        #     if use_extended_form
        #       # set new sheet with extended form
        #       ctx.view_template.data_uri = UU::OS::UESURIBuilder.parse_uesuri(UCR_URI).set_object_code(EXTENDED_FORM_CODE).to_uesuri
        #     end
        #   
        #     # if you do not set data_uri so default is used
        #   
        #   end
        #   
        #
        # @return [String]
        attr_accessor :data_uri
        
        
        def initialize(data = nil)
          if data.nil?
            @raw_attrs = {}
            return
          end
          if data.kind_of?String
            data = JSON.parse(data, :symbolize_names => true)
          end
          if data.kind_of?Hash
            @raw_attrs = data
          elsif data.kind_of?ViewTemplate
            if data.raw_attrs
              @raw_attrs = data.raw_attrs
            else
              @raw_attrs = {}
            end
          else
            raise ArgumentError.new("ViewTemplate data must be JSON String or Hash, but was #{data.class}.")
          end
        end

        def data_uri
          @raw_attrs[:dataUri]
        end

        def data_uri=(data_uri)
          @raw_attrs[:dataUri] = data_uri
        end

      end
    end
  end
end